@@ -1,4 +1,6 @@
+import StringIO
 import os
+import xlrd
 
 from django import forms
 from django.utils.translation import ugettext_lazy as _
@@ -6,15 +8,34 @@ from django.utils.translation import ugettext_lazy as _
 IMPORT_FILE_TYPES = ['.xls', ]
 
 class XlsInputForm(forms.Form):
-    input_excel = forms.FileField(required= True, label= u"Upload the Excel file to import to the system.")
+    input_excel = forms.FileField(required=True,
+        label=_("Upload the Excel file to import to the system."))
 
-    def clean_input_excel(self):
-        input_excel = self.cleaned_data['input_excel']
+    def clean(self):
+        data = super(XlsInputForm, self).clean()
+
+        if 'input_excel' not in data:
+            raise forms.ValidationError(_('The Excel file is required to proceed.'))
+
+        input_excel = data['input_excel']
         extension = os.path.splitext( input_excel.name )[1]
         if not (extension in IMPORT_FILE_TYPES):
-            raise forms.ValidationError( u'%s is not a valid excel file. Please make sure your input file is an excel file (Excel 2007 is NOT supported.' % extension )
-        else:
-            return input_excel
+            raise forms.ValidationError(
+                _(u'%s is not a valid Excel file. Please make sure your input file is an Excel file (Excel 2007 is NOT supported.)') % input_excel.name)
+
+        file_data = StringIO.StringIO()
+        for chunk in input_excel.chunks():
+            file_data.write(chunk)
+        data['file_data'] = file_data.getvalue()
+        file_data.close()
+
+        try:
+            xlrd.open_workbook(file_contents=data['file_data'])
+        except xlrd.XLRDError, e:
+            raise forms.ValidationError(_('Unable to open XLS file: %s' % e))
+
+        return data
+
 
 class ColumnAssignForm(forms.Form):
     def __init__(self, *args, **kwargs):
